#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""PAL command for processing the XML configuration file."""


from __future__ import with_statement

import os
import errno

import iobase.xmldata as xmldata
import iobase.fileut as fileut

import libpal.palcmd as palcmd
import pal_config as config
from pal_info import (ReqTypes, Results)


class PalConffile(palcmd.PalCmd):
    """Pal conffile implementation class."""

    def __load_handler(self, entry):
        """Handler for "load" request."""

        file_name = os.path.join(config.XML_CONF_DIR, entry.get_file())

        with fileut.FLock(config.LOCK_DIR, os.path.basename(file_name),
                            exclusive=False):

            try:
                xml_data = xmldata.XmlData(file_name)
                entry.set(xml_data.get_root_node())
                entry.set_result(Results.OK)
            except IOError, err:
                # "err.errno" is set to None, when using "lxml.etree"
                if err.errno == errno.ENOENT or err.errno == None:
                    entry.set_result(Results.FILE_NOT_FOUND)
                else:
                    entry.set_result(Results.OPEN_ERROR)

        return entry

    def __save_handler(self, entry):
        """Handler for "save" request."""

        file_name = os.path.join(config.XML_CONF_DIR, entry.get_file())

        with fileut.FLock(config.LOCK_DIR, os.path.basename(file_name),
                            exclusive=True):

            try:
                xml_data = xmldata.XmlData(entry.get())
                xml_data.save(file_name)
                entry.set_result(Results.OK)
            except IOError, err:
                # "err.errno" is set to None, when using "lxml.etree"
                if err.errno == errno.ENOENT or err.errno == None:
                    entry.set_result(Results.FILE_NOT_FOUND)
                else:
                    entry.set_result(Results.OPEN_ERROR)

        return entry

    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.LOAD, self.__load_handler)
        self.set_handler(ReqTypes.SAVE, self.__save_handler)


#
# Main
#

if __name__ == "__main__":
    PalConffile(option=palcmd.PalCmdOption()).handle_request()
