#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from libpal import palcmd
from libpal.util import RemountRoot
from pal_info import ReqTypes, Results
from iobase.exception import ExecError, InvalidArgError
from iobase.cmdexec import CmdExec, NULL, PIPE
import pal_config as config


class PalQRcode(palcmd.PalCmd):
    def __encode_handler(self, entry):
        """
        <entry type="encode" file="output_file_path">
            <qrcode>
                <data>encode_string</data>
            </qrcode>
        </entry>
        """
        try:
            qrcode_node = entry.get()
            string = qrcode_node.get_value('data', None)
            if string is None:
                raise  InvalidArgError('Data is empty!!')
            try:
                fpath = entry.get_file()
            except:
                raise  InvalidArgError('File path is empty!!')

            cmd = [config.QRENCODE_CMD]  # FIXME qrencode
            # 必要ならoptionをここで追加する
            cmd = cmd + ['-o', fpath] 
            cmd = cmd + [string]
            with RemountRoot():
                CmdExec(cmd).execute(stdout=NULL, stderr=NULL)

            entry.set_result(Results.OK)
        except:
            entry.set_result(Results.CREATE_ERROR)
            self.exception('EncodeError')  # FIXME msg

        return entry

    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.ENCODE, self.__encode_handler)

#
# Main
#

if __name__ == "__main__":
    PalQRcode(option=palcmd.PalCmdOption()).handle_request()
