#!/bin/bash
### BEGIN INIT INFO
# Provides:          remotelink3
# Required-Start:
# Required-Stop:
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 6
# Short-Description: remotelink3
# Description:       Remote Link 3 daemon
### END INIT INFO

BINDIR=/opt/rl3/bin
RUNDIR=/var/run
TOU_SERVER=${BINDIR}/tou_server
TOU_SERVER_PIDFILE=${RUNDIR}/tou_server.pid
DEVICE_PROCESS=${BINDIR}/device_process
DEVICE_PROCESS_PIDFILE=${RUNDIR}/device_process.pid
LOGD=${BINDIR}/logd
LOGD_PIDFILE=${RUNDIR}/rl3_logd.pid
WATCHDOGD=${BINDIR}/watchdogd.sh
WATCHDOGD_PIDFILE=${RUNDIR}/rl3_watchdogd.pid

LOG_FIFO=/var/log/rl3.log.fifo

PATH=/sbin:/usr/sbin:/bin:/usr/bin:/usr/local/bin

function get_pid() {
	local cmd_path="$1"
	local pid_file="$2"
	local name=`basename ${cmd_path}`

	if [ -e ${pid_file} ]; then
		local pid=`cat ${pid_file}`
		local comm=`ps --no-header -o comm -p ${pid}` 

		if [ x"${name}" = x"${comm}" ]; then
			echo ${pid}
			return
		fi

		local greped_pid=`pgrep -f ${cmd_path}`
		if [ x"${greped_pid}" = x"${pid}" ]; then
			echo ${pid}
			return
		fi
	fi

	ps --no-header -o pid -C ${name}
}

function start_process() {
	local cmd_path="$1"
	local pid_file="$2"
	local do_daemonize=$3
	local name=`basename ${cmd_path}`

	if [ x"`get_pid ${cmd_path} ${pid_file}`" != x"" ]; then
		echo "${name} process already running..."
		return 0
	fi

	echo "${name} process starting..."
	if [ ${do_daemonize} -ne 0 ]; then
		start-stop-daemon --start --quiet --oknodo --background \
			--pidfile ${pid_file} --make-pidfile \
			--exec ${cmd_path}
	else
		${cmd_path} & > /dev/null 2>&1
	fi
}

function stop_process() {
	local cmd_path="$1"
	local pid_file="$2"
	local do_daemonize=$3
	local retry_count=$4
	local name=`basename ${cmd_path}`
	local pid=`get_pid ${cmd_path} ${pid_file}`

	if [ x"${pid}" = x"" ]; then
		echo "${name} process does not running..."
		return
	fi

	echo "${name} process stoping..."
	if [ ${do_daemonize} -ne 0 ]; then
		start-stop-daemon --stop --quiet --pidfile ${pid_file}
		rm -f ${pid_file}
	else
		kill -15 ${pid} 2>/dev/null
	fi

	if [ x"${retry_count}" = x"" ]; then
		retry_count=5
	fi

	for count in `seq 0 ${retry_count}`; do
		pid=`get_pid ${cmd_path} ${pid_file}`

		if [ x"${pid}" = x"" ]; then
			return
		fi
		sleep 1
	done

	echo "killing ${name} process..."
	kill -9 ${pid} 2>/dev/null
}

function start() {
	start_process ${LOGD} ${LOGD_PIDFILE} 1
	start_process ${WATCHDOGD} ${WATCHDOGD_PIDFILE} 1
	start_process ${TOU_SERVER} ${TOU_SERVER_PIDFILE} 0
	start_process ${DEVICE_PROCESS} ${DEVICE_PROCESS_PIDFILE} 0
}


function stop() {
	stop_process ${WATCHDOGD} ${WATCHDOGD_PIDFILE} 1
	stop_process ${TOU_SERVER} ${TOU_SERVER_PIDFILE} 0
	stop_process ${DEVICE_PROCESS} ${DEVICE_PROCESS_PIDFILE} 0 30
	stop_process ${LOGD} ${LOGD_PIDFILE} 1
}


function usage() {
  echo "$0 < start | stop | usage >"
}


function restart() {
	stop_process ${DEVICE_PROCESS} ${DEVICE_PROCESS_PIDFILE} 0 30
	start_process ${DEVICE_PROCESS} ${DEVICE_PROCESS_PIDFILE} 0
}

########################################
# main logic
########################################

rm -f $LOG_FIFO
mkfifo $LOG_FIFO

case $1 in
	"start" ) 
		start
		;;
	"stop" )
		stop
		;;
	"restart" )
		echo "restart only called by device_process or tou_server"
		restart
		;;
	* )
		usage
		;;
esac

# end of file.
