# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
import glob
import time
import re
import uuid
import errno
import subprocess as sp

import rapsexcept

from stat import S_ISDIR
from stat import ST_MTIME

FILELIST_END = "\n\n"


class rapssession:

    SESSION_PATH = ""
    LIST_PATH = ""
    SESSION_EXT  = ".ses"
    FILELST_EXT  = ".lst"
    TMPFILELST_EXT  = ".tmplst"
    MAX_MEMORY   = 0

    c_comm = None

    def __init__(self, comm):
        '''
        Constructor
        '''
        self.c_comm = comm

    def setConfig(self):
        '''
        setting config values
        '''
        if self.c_comm == None:
            return
        if self.c_comm.CONFIG == None:
            return
        self._setSessionPath(self.c_comm.CONFIG.SESS_PATH)
        self._setListPath(self.c_comm.CONFIG.LIST_PATH, self.c_comm.CONFIG.MAX_MEMORY )
        return

    def _setSessionPath(self,path):
        '''
        setting session path
        '''
        self.SESSION_PATH = path.rstrip("/")
        self.SESSION_PATH += "/"
        return

    def _setListPath(self,path, maxmem):
        '''
        setting  list path
        '''
        self.LIST_PATH  = path.rstrip("/")
        self.LIST_PATH += "/"
        self.MAX_MEMORY = maxmem
        return

    def isCount(self):
        '''
        session file count
        '''
        lst = glob.glob( self._getfname( "*" ) )
        return len(lst)
        
    def create(self, session, sessdata):
        '''
        cleate session file
        '''
        n_result = self.c_comm.RESULT.RESULT_NO_ERROR

        # remove
        self.remove()

        # create session file
        try:
            fp = open( self._getfname(session), "w" )
            fp.write(sessdata)
            fp.close()

        except (IOError, OSError), e:
            if e.errno == errno.EACCES:
                n_result = self.c_comm.RESULT.RESULT_ERR_PERMISSION
            elif e.errno == errno.ENOSPC:
                n_result = self.c_comm.RESULT.RESULT_ERR_FILESYSTEM_FULL
            else:
                n_result = self.c_comm.RESULT.RESULT_ERR_WRITE

        except:
            n_result = self.c_comm.RESULT.RESULT_ERR_WRITE

        # parmission set
        try:
            os.chmod( self._getfname(session), 0666 )
        except:
            pass

        return n_result

    def delete(self,session):
        '''
        delete session file
        '''
        n_result = self.c_comm.RESULT.RESULT_NO_ERROR
        try:
            # session
            os.remove( self._getfname(session) )

        except (IOError, OSError), e:
            if  e.errno == errno.ENOENT:
                n_result = self.c_comm.RESULT.RESULT_NOT_FOUND
            elif e.errno == errno.EACCES:
                n_result = self.c_comm.RESULT.RESULT_ERR_PERMISSION
            else:
                n_result = self.c_comm.RESULT.RESULT_ERR_DELETE

        try:
            # file list
            os.remove( self._getlstname(session) )
        except:
            pass

        return n_result

    def remove( self, session="" ):
        '''
        remove session file
        '''
        n_result = self.c_comm.RESULT.RESULT_NO_ERROR
        if session == "":
            try:
                l_files = os.listdir( self.SESSION_PATH )
                for s_name in l_files:
                    s_file  = os.path.join( self.SESSION_PATH, s_name )
                    n_time  = os.stat( s_file )[ST_MTIME]
                    n_passd = int(time.time( )) - n_time
                    if n_passd > self.c_comm.CONFIG.TIME_SESSION:
                        # parse file name
                        s_session, s_ext = os.path.splitext(s_name)
                        if s_ext.lower() == self.SESSION_EXT:
                            # remove session
                            result = self.delete( s_session )
                            if n_result != self.c_comm.RESULT.RESULT_NO_ERROR:
                                n_result = result
            except:
                n_result = self.c_comm.RESULT.RESULT_ERR_DELETE

        else:
            try:
                n_time  = os.stat( self._getfname(session) )[ST_MTIME]
                n_passd = int(time.time( )) - n_time
                if n_passd > self.c_comm.CONFIG.TIME_SESSION:
                    # remove session
                    n_result = self.delete( session )
            except:
                n_result = self.c_comm.RESULT.RESULT_ERR_DELETE

        return n_result

    def check( self, session ):
        '''
        session check
        '''
        retCode = False

        # remove
        self.remove(session)

        # check
        try:
            fp = open( self._getfname(session), "r" )
            fp.close( )
            retCode = True
        except:
            pass

        if retCode == True:
            self._updatetime(session)
        
        return retCode

    def update( self, session ):
        '''
        session update
        '''
        if session != "":
            self._updatetime(session)
        
        return

    def getSessionData( self, session ):
        '''
        user name is acquired. 
        '''
        retData = ""
        try:
            fp = open( self._getfname(session), "r" )
            retData = fp.read()
            fp.close( )
        except:
            pass
        
        return retData

    def _updatetime(self, session):
        '''
        time stamp is updated. 
        '''
        try:
            os.utime( self._getfname(session), None )
        except:
            pass

    def check_create_filelist(self, session, index,  path, regex, sort, order, precision, recurrence):
        '''
        It confirms whether create a file list.
        '''

        # initalize
        s_sname    = self._getlstname(session)
        f_makelist = False
        s_path     = ""
        s_regex    = ""
        s_sort     = ""
        b_order    = False
        n_prec     = 0
        b_recu     = False

        # index zero check
        if index == 0:
            f_makelist = True
            return f_makelist

        # open file list
        try:
            fp = open( s_sname, "r" )
        except:
            f_makelist = True
            return f_makelist

        # The line of a search condition is skipped. 
        conditions = fp.readline()
        if conditions != "":
            try:
                conditions = conditions.rstrip("\n")
                s_path, s_regex, s_sort, b_order, n_prec, b_recu, n_total = self._parse_conditions(conditions)
            except:
                pass

        # file close
        fp.close()

        # The check of change 
        s1 = path.rstrip("/")
        s2 = s_path.rstrip("/")
        if s1 != s2:
            f_makelist = True
        if regex != s_regex: 
            f_makelist = True
        if sort != s_sort: 
            f_makelist = True
        if order != b_order: 
            f_makelist = True
        #if precision != n_prec: 
        #    f_makelist = True
        if recurrence != b_recu: 
            f_makelist = True
        return f_makelist

    def getlistcount(self, session, path, recurrence, maxcount):
        '''
        get filelist count
        '''
        # initalize
        s_sname = self._getlstname(session)
        s_path  = ""
        s_regex = ""
        s_sort  = ""
        b_order = False
        n_prec  = 0
        b_recu  = False
        b_check = False 
        n_count = 0

        # open file list
        try:
            fp = open( s_sname, "r" )
        except:
            return 0

        # The line of a search condition is skipped. 
        conditions = fp.readline()
        if conditions != "":
            try:
                conditions = conditions.rstrip("\n")
                s_path, s_regex, s_sort, b_order, n_prec, b_recu, n_count = self._parse_conditions(conditions)
            except:
                pass

        # The check of change 
        s1 = path.rstrip("/")
        s2 = s_path.rstrip("/")
        if s1 != s2:
            b_check = True
        if ".*" != s_regex: 
            b_check = True
        if "none" != s_sort: 
            b_check = True
        if recurrence != b_recu: 
            b_check = True

        # file close
        fp.close()

        return n_count

    def _adjust_fp_line_pointer(self, tmpfp, path):

        """
        findの結果の1行目は
        findの根元のpathである場合と
        そうでない場合があるので調節
        """
        line = tmpfp.readline().strip("\n")
        if path != line:
            tmpfp.seek(0)
        
    def _get_find_cmd(self, path, regex):
        if regex == ".*":
            return ["find", path]

        """
        regexが指定されている場合、
        findの-regexオプションを指定することで
        findの結果の量を減らし後の処理量を削減できる。
        そのための変換処理。
        """
        if regex.startswith("^"):
            regex = regex[1:]
        return ["find", path, "-regex", ".*" + regex]

    def _find(self, path, fname, regex):
        
        fp = open(fname, "w")
        os.chmod( fname, 0666 )
        proc = sp.Popen(self._get_find_cmd(path, regex), stdout=fp)
        result = proc.wait()
        fp.close()
        if result != 0:
            raise rapsexcept.ListError

    def create_filelist(self, session, path, regex, sort, order, precision, recurrence):
        '''
        create_filelist
        '''
        # initalize
        s_sname    = self._getlstname(session)
        s_tmpsname = self._gettmplstname(session)
        n_count = 0
        n_size  = 0
        r_req = None

        if len(regex) > 0 and regex != ".*":
            r_req = re.compile(regex)

        path  = path.rstrip("/")
        path += "/"

        # create file list
        fp = open( s_sname, "w" )
        os.chmod( s_sname, 0666 )

        # It searches.
        if recurrence == False:
            # 先に全filelistをmemoryへ取得し、conditions行を書き出す
            all_list = []
            for name in os.listdir( path ):
                # check regex
                if r_req != None:
                    if r_req.match(name) == None:
                        continue
                all_list.append([path, name])

            n_count = len(all_list) 
            conditions = self._make_conditions(path, regex, sort, order, precision, recurrence, n_count)

            fp.write(conditions)

            # filelist作成実処理
            for path, name in all_list:
                self._add_filelist( fp, path, name, precision ) 

        else:
            try:
                # pythonのos.walkは遅いのでfindコマンドを使用
                self._find(path, s_tmpsname, regex)
                tmpfp = open(s_tmpsname)

                self._adjust_fp_line_pointer(tmpfp, path)

                # 先にn_countを取得しconditions行を書き出す
                for line in tmpfp:
                    # check regex
                    if r_req != None:
                        striped = line.rstrip(os.path.sep + "\n")
                        name = striped.split(os.path.sep)[-1]
                        if r_req.match(name) == None:
                            continue
                    n_count += 1
                conditions = self._make_conditions(path, regex, sort, order, precision, recurrence, n_count)

                fp.write(conditions)

                tmpfp.seek(0)

                self._adjust_fp_line_pointer(tmpfp, path)

                # filelist作成実処理
                for line in tmpfp:
                    striped = line.rstrip(os.path.sep + "\n")
                    splited = striped.split(os.path.sep)

                    name = splited.pop()
                    par = os.path.sep.join(splited)

                    # check regex
                    if r_req != None:
                        if r_req.match(name) == None:
                            continue

                    self._add_filelist( fp, par, name, precision )


            finally:
                try:
                    tmpfp.close()
                    os.unlink(s_tmpsname)
                except:
                    pass

        fp.write(FILELIST_END)

        # close.
        fp.close()

        # sort
        self._sort_filelist( s_sname, sort, order )
        
        return n_count

    def get_filelist(self, session, index, maxnum ):
        '''
        get_filelist
        '''
        # initalize
        s_sname    = self._getlstname(session)
        n_count    = 0
        n_maxcount = 0
        n_line     = 0
        l_item     = []

        # open file list
        try:
            fp = open( s_sname, "r" )
        except:
            return l_item, n_count, n_maxcount
        
        # The line of a search condition is skipped. 
        buff = fp.readline()
        if buff == "":
            return l_item, n_count, n_maxcount
        else:
            try:
                buff = buff.rstrip("\n")
                s_path, s_regex, s_sort, b_order, n_prec, b_recu, n_total = self._parse_conditions(buff)
            except:
                pass

         
        # read file list
        buff = fp.readline().rstrip("\n")
        while buff != "":
            if n_line >= index:
                # maxnum over
                if maxnum > 0:
                    if n_count >= maxnum:
                        break
                
                # append list
                buff += "\t" + str(n_line)
                try:
                    l_item.append(buff)
                except MemoryError:
                    raise
                n_count += 1
                
            n_line     += 1
            n_maxcount += 1
            buff = fp.readline().rstrip("\n")

        
        # close.
        fp.close()

        if n_count < maxnum:
            # n_count < maxnumということは
            # filelist分割取得の最後尾リクエストと見なす
            if index + n_count != n_total:
                # n_totalと実際のfilelist数が一致しない場合
                raise rapsexcept.ListError([l_item, n_count, n_total])

        return l_item, n_count, n_total

    def _add_filelist(self, fp, path, name, precision ):
        '''
        add_filelist
        '''
        fname = os.path.join( path, name )
        try:
            fst = os.stat( fname )
            if fst == None:
                return 0
        except:
            return 0

        # mtime
        output_fmt = "%Y%m%d%H%M%S"
        time_fmt = time.strftime(output_fmt,time.gmtime(fst.st_mtime))
        msec = repr(os.path.getmtime(fname)).split(".")[1] 

        # add file list
        buff = path + "\t" + name + "\t" + str(fst.st_size) + "\t" + time_fmt + "." + msec + "\t"
        if S_ISDIR(fst.st_mode) == True:
            buff += "1"
        else:
            buff += "0"
        buff += "\n"
        fp.write( buff )
        fp.flush()

        return len(buff)

    def _sort_filelist(self, sname, sort, _reverse=False):
        '''
        sort file list
        '''
        # initalize
        lines = []
        
        # sort pattern'
        if sort == "none":
            return True
        elif sort == "name":
            keynum = 1
        elif sort == "size":
            keynum = 2
        elif sort == "mtime":
            keynum = 3
        else:
            keynum = 1

        # sorting
        try:
            # read filelist
            fp = open( sname, "r" ) 
            
            # The 1st line is an extraction condition.
            conditions = fp.readline()
            if conditions == "":
                return False
            
            # The 2nd line or subsequent ones is sorted
            lines = fp.readlines()

            # 完成されたlistだと末尾2行が改行のみなので
            # 後のsort処理のため、末尾2行が改行ならばそれらをpopする
            if lines[-1] == "\n":
                lines.pop()
            if lines[-1] == "\n":
                lines.pop()

            if keynum != 2:
                # Except size.
                lines.sort(cmp=lambda x,y: cmp( str(x).split("\t")[keynum], str(y).split("\t")[keynum] ), reverse=_reverse)
            else:
                # size.
                lines.sort(cmp=lambda x,y: cmp( long(str(x).split("\t")[keynum]), long(str(y).split("\t")[keynum]) ), reverse=_reverse)
            fp.close()

        except MemoryError:
            raise
        
        except:
            return False

        # writing
        try:
            # read filelist
            fp = open( sname, "w" ) 
            fp.write(conditions)
            fp.writelines(lines)
            fp.write(FILELIST_END)
            fp.close()
        except:
            return False

        return True

    def delete_filelist(self, session):
        '''
        delete_filelist
        '''
        __path = ""
        __regex = ""
        
        # file list
        try:
            os.remove( self._getlstname(session) )
        except:
            pass
    
    def _getfname(self, session):
        '''
        _getfname
        '''
        return self.SESSION_PATH + session + self.SESSION_EXT
    
    def _getlstname(self, session):
        '''
        _getlstname
        '''
        return self.LIST_PATH + session + self.FILELST_EXT

    def _gettmplstname(self, session):
        '''
        _gettmplstname
        '''
        return self.LIST_PATH + session + self.TMPFILELST_EXT
    
    def _make_conditions(self,  path, regex, sort, order, precision, recurrence, count):
        '''
        make conditions
        '''
        if order == False:
            s_order = "0"
        else:
            s_order = "1"
        try:
            s_prec = str(precision)
        except:
            s_prec = "0"
        if recurrence == False:
            s_recu = "0"
        else:
            s_recu = "1"
        try:
            s_cnt = str(count)
        except:
            s_cnt = "0"

        
        return path + "\t" + regex + "\t" + sort + "\t" + s_order + "\t" + s_prec + "\t" + s_recu + "\t" + s_cnt + "\n"
    
    def _parse_conditions(self, conditions):
        '''
        parse conditions
        '''
        path  = ""
        regex = ""
        sort  = ""
        order = False
        prec  = 0
        recu  = False
        
        try:
            buff = conditions.split("\t")
        except:
            pass
        try:
            path = buff[0]
        except:
            pass
        try:
            regex = buff[1]
        except:
            pass
        try:
            sort = buff[2]
        except:
            pass
        try:
            if buff[3] == "1":
                order = True
        except:
            pass
        try:
            prec = int(buff[4])
        except:
            pass
        try:
            if buff[5] == "1":
                recu = True
        except:
            pass
        try:
            count = int(buff[6])
        except:
            pass
        
        return path, regex, sort, order, prec, recu, count

    def make_filelist(self, l_item, maxcount, index, pathlen, b_size, b_mtime, precision, target_file_list=[]):
        '''
        make filelist
        '''
        n_count = 0
        s_filelist = ""
        for row in l_item:
            try:
                items = row.split("\t") 
                if len(items) >= 5:
                    if (len(target_file_list) and
                        items[1] not in target_file_list):

                        maxcount -= 1
                        continue

                    n_count += 1
                    path  = items[0]
                    name  = items[1]
                    size  = items[2]
                    date  = items[3].split(".")
                    mtim  = date[0]
                    try:
                        msec  = date[1]
                    except:
                        msec  = ""
                        pass
                    isdir = items[4]
                    #num   = items[5]
                    if isdir != "1":
                        s_filelist += "  <file"
                    else:
                        s_filelist += "  <file isdir=\x22true\x22"
                    if b_size == True:
                        s_filelist += " size=\x22"
                        s_filelist += size
                        s_filelist += "\x22"
                    if b_mtime == True:
                        s_filelist += " mtime=\x22"
                        s_filelist += mtim
                        if precision > 0:
                            lsec = len(msec)
                            if precision > lsec:
                                msec += "0" * (precision - lsec)
                            elif precision < lsec:
                                msec = msec[0:precision] 
                            s_filelist += msec
                        s_filelist += "\x22"
                    s_filelist += ">\n"
                    s_filelist += "   <name>"
                    s_filelist += self.c_comm.Escape(name)
                    s_filelist += "</name>\n"
                    if pathlen > 0:
                        s_filelist += "   <directory>"
                        s_filelist += self.c_comm.Escape(path[pathlen:].rstrip("/"))
                        s_filelist += "</directory>\n"
                    s_filelist += "  </file>\n"
            
            except MemoryError:
                raise
            except:
                pass

        try:
            s_filelist += " </filelist>" 
        except MemoryError:
            raise
        
        # filelist header
        s_hfile = " <filelist total=\x22%d\x22 index=\x22%d\x22>\n" % (maxcount, index) 
        
        # make file list
        s_filelist = s_hfile + s_filelist
        
        return s_filelist

    def getSeesionid(self):
        '''
        creat session id
        '''
        return str(uuid.uuid4())
    
