#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Status information module."""


from fal_msginfo import (Cat, get_category_str)
from ionas.request import (Entry, Request)
from fal_info import (FAL_STATEMGR_PATH, FAL_RAIDCTRL_PATH)
from product_conf import (SUDO_CMD,)
from product_conf.reqinfo import (ReqTypes, Results)


STATEMGR_NAME = "statemgr"
STATE_PATH = "statelist/state"
CATEGORY_ATTR = "category"

TYPE_INFO_CODE = 0
TYPE_PROGRESS_CODE = 1
TYPE_RECOVER_CODE = 2
TYPE_WARNING_CODE = 3
TYPE_ERROR_CODE = 4
TYPE_FATAL_ERROR_CODE = 5

TYPE_INFO_STRING = "情報"
TYPE_PROGRESS_STRING = "進捗"
TYPE_RECOVER_STRING = "復帰"
TYPE_WARNING_STRING = "警告"
TYPE_ERROR_STRING = "エラー"
TYPE_FATAL_ERROR_STRING = "致命的エラー"


SYSTEM_CATEGORY_CODE = int(Cat.SYSTEM)
RAID_CATEGORY_CODE = int(Cat.RAID)
RAID_VOLUME_CATEGORY_CODE = int(Cat.RAID_VOLUME)
VOLUME_CATEGORY_CODE = int(Cat.VOLUME)
ACTIVE_REPAIR_CATEGORY_CODE = int(Cat.ACTIVE_REPAIR)
BACKUP_CATEGORY_CODE = int(Cat.BACKUP)
QUICKCOPY_CATEGORY_CODE = int(Cat.QUICKCOPY)

SYSTEM_CATEGORY_STRING = get_category_str(Cat.SYSTEM)
RAID_CATEGORY_STRING = get_category_str(Cat.RAID)
RAID_VOLUME_CATEGORY_STRING = get_category_str(Cat.RAID_VOLUME)
VOLUME_CATEGORY_STRING = get_category_str(Cat.VOLUME)
ACTIVE_REPAIR_CATEGORY_STRING = get_category_str(Cat.ACTIVE_REPAIR)
BACKUP_CATEGORY_STRING = get_category_str(Cat.BACKUP)
QUICKCOPY_CATEGORY_STRING = get_category_str(Cat.QUICKCOPY)

RAID_PROGRESS_PATH = "raid/progress"
QUERY_RAIDCTRL_XML_TMPL = """<raidctrl>
        <raid depth="%d"><name type="volume">%s</name></raid>
    </raidctrl>"""

STATUS_INFO = {
        "system": {
                "booting": {
                        "type": TYPE_INFO_CODE,
                        "type_string": TYPE_INFO_STRING,
                        "category": SYSTEM_CATEGORY_CODE ,
                        "category_string": SYSTEM_CATEGORY_STRING ,
                        "status": SYSTEM_CATEGORY_CODE * 10000 + 1,
                        "status_string": "起動中",
                    },
                "busy": {
                        "type": TYPE_INFO_CODE,
                        "type_string": TYPE_INFO_STRING,
                        "category": SYSTEM_CATEGORY_CODE ,
                        "category_string": SYSTEM_CATEGORY_STRING ,
                        "status": SYSTEM_CATEGORY_CODE * 10000 + 2,
                        "status_string": "処理中",
                    },
                "updating": {
                        "type": TYPE_INFO_CODE,
                        "type_string": TYPE_INFO_STRING,
                        "category": SYSTEM_CATEGORY_CODE ,
                        "category_string": SYSTEM_CATEGORY_STRING ,
                        "status": SYSTEM_CATEGORY_CODE * 10000 + 3,
                        "status_string": "システム更新中",
                    },
                "shutdown": {
                        "type": TYPE_INFO_CODE,
                        "type_string": TYPE_INFO_STRING,
                        "category": SYSTEM_CATEGORY_CODE ,
                        "category_string": SYSTEM_CATEGORY_STRING ,
                        "status": SYSTEM_CATEGORY_CODE * 10000 + 4,
                        "status_string": "シャットダウン中",
                    },
            },
        "raid": {
                "preparing": {
                        "type": TYPE_INFO_CODE,
                        "type_string": TYPE_INFO_STRING,
                        "category": RAID_CATEGORY_CODE ,
                        "category_string": RAID_CATEGORY_STRING ,
                        "status": RAID_CATEGORY_CODE * 10000 + 1,
                        "status_string": "準備中",
                    },
                "stop": {
                        "type": TYPE_INFO_CODE,
                        "type_string": TYPE_INFO_STRING,
                        "category": RAID_CATEGORY_CODE ,
                        "category_string": RAID_CATEGORY_STRING ,
                        "status": RAID_CATEGORY_CODE * 10000 + 2,
                        "status_string": "停止中",
                    },
                "degrade": {
                        "type": TYPE_ERROR_CODE,
                        "type_string": TYPE_ERROR_STRING,
                        "category": RAID_CATEGORY_CODE ,
                        "category_string": RAID_CATEGORY_STRING ,
                        "status": RAID_CATEGORY_CODE * 10000 + 3,
                        "status_string": "構成異常",
                    },
                "fail": {
                        "type": TYPE_ERROR_CODE,
                        "type_string": TYPE_ERROR_STRING,
                        "category": RAID_CATEGORY_CODE ,
                        "category_string": RAID_CATEGORY_STRING ,
                        "status": RAID_CATEGORY_CODE * 10000 + 4,
                        "status_string": "異常",
                    },
                "crash": {
                        "type": TYPE_FATAL_ERROR_CODE,
                        "type_string": TYPE_FATAL_ERROR_STRING,
                        "category": RAID_CATEGORY_CODE ,
                        "category_string": RAID_CATEGORY_STRING ,
                        "status": RAID_CATEGORY_CODE * 10000 + 4,
                        "status_string": "崩壊",
                    },
            },
        "raidvol": {
                "preparing": {
                        "type": TYPE_INFO_CODE,
                        "type_string": TYPE_INFO_STRING,
                        "category": RAID_VOLUME_CATEGORY_CODE ,
                        "category_string": RAID_VOLUME_CATEGORY_STRING ,
                        "status": RAID_VOLUME_CATEGORY_CODE * 10000 + 1,
                        "status_string": "準備中",
                    },

                "rebuilding": {
                        "type": TYPE_PROGRESS_CODE,
                        "type_string": TYPE_PROGRESS_STRING,
                        "category": RAID_VOLUME_CATEGORY_CODE ,
                        "category_string": RAID_VOLUME_CATEGORY_STRING ,
                        "status": RAID_VOLUME_CATEGORY_CODE * 10000 + 2,
                        "status_string": "再構築中",
                        "progress": 0,
                    },
                "fail": {
                        "type": TYPE_ERROR_CODE,
                        "type_string": TYPE_ERROR_STRING,
                        "category": RAID_VOLUME_CATEGORY_CODE,
                        "category_string": RAID_VOLUME_CATEGORY_STRING,
                        "status": RAID_VOLUME_CATEGORY_CODE * 10000 + 3,
                        "status_string": "異常",
                    },
                "mountfail": {
                        "type": TYPE_FATAL_ERROR_CODE,
                        "type_string": TYPE_FATAL_ERROR_STRING,
                        "category": RAID_VOLUME_CATEGORY_CODE,
                        "category_string": RAID_VOLUME_CATEGORY_STRING,
                        "status": RAID_VOLUME_CATEGORY_CODE * 10000 + 4,
                        "status_string": "マウント失敗",
                    },
                "keywaiting": {
                        "type": TYPE_INFO_CODE,
                        "type_string": TYPE_INFO_STRING,
                        "category": RAID_VOLUME_CATEGORY_CODE,
                        "category_string": RAID_VOLUME_CATEGORY_STRING,
                        "status": RAID_VOLUME_CATEGORY_CODE * 10000 + 5,
                        "status_string": "セキュリティロック中",
                    },
            },
        "filesystem": {
                "formatting": {
                        "type": TYPE_INFO_CODE,
                        "type_string": TYPE_INFO_STRING,
                        "category": VOLUME_CATEGORY_CODE,
                        "category_string": VOLUME_CATEGORY_STRING,
                        "status": VOLUME_CATEGORY_CODE * 10000 + 1,
                        "status_string": "フォーマット中",
                    },
                "checking": {
                        "type": TYPE_INFO_CODE,
                        "type_string": TYPE_INFO_STRING,
                        "category": VOLUME_CATEGORY_CODE,
                        "category_string": VOLUME_CATEGORY_STRING,
                        "status": VOLUME_CATEGORY_CODE * 10000 + 2,
                        "status_string": "チェックディスク中",
                    },
            },
        "activerepair": {
                "running": {
                        "type": TYPE_PROGRESS_CODE,
                        "type_string": TYPE_PROGRESS_STRING,
                        "category": ACTIVE_REPAIR_CATEGORY_CODE,
                        "category_string": ACTIVE_REPAIR_CATEGORY_STRING,
                        "status": ACTIVE_REPAIR_CATEGORY_CODE * 10000 + 1,
                        "status_string": "アクティブリペア中",
                        "progress": 0,
                    },
            },
        "backup": {
                "running": {
                        "type": TYPE_INFO_CODE,
                        "type_string": TYPE_INFO_STRING,
                        "category": BACKUP_CATEGORY_CODE,
                        "category_string": BACKUP_CATEGORY_STRING,
                        "status": BACKUP_CATEGORY_CODE * 10000 + 1,
                        "status_string": "バックアップ中",
                    },
            },
        "quickcopy": {
                "running": {
                        "type": TYPE_INFO_CODE,
                        "type_string": TYPE_INFO_STRING,
                        "category": QUICKCOPY_CATEGORY_CODE,
                        "category_string": QUICKCOPY_CATEGORY_STRING,
                        "status": QUICKCOPY_CATEGORY_CODE * 10000 + 1,
                        "status_string": "クイックコピー中",
                    },
            },
    }


def _request(fal_cmd, entry):
    response = Request(fal_cmd, filter_cmd=SUDO_CMD).request(entry)
    return response.get_entrylist().get_entry().get()


def _get_rebuild_progress(depth, volume):

    try:
        raidctrl_node = _request(FAL_RAIDCTRL_PATH,
                            Entry(ReqTypes.QUERY,
                                QUERY_RAIDCTRL_XML_TMPL % (depth, volume)))
        progress = raidctrl_node.get_node(RAID_PROGRESS_PATH).get_text()
        return progress
    except:
        # Ignore error
        pass

    return None


def _set_rebuild_progress(status_info):

    progress = _get_rebuild_progress(2, "sataraid1")
    if progress is None:
        progress = _get_rebuild_progress(1, "sataraid1")
    if progress is None:
        progress = "0"

    status_info["status_string"] += "(%s%%)" % (progress,)
    status_info["progress"] = int(progress)


def _set_active_repair_progress(status_info):

    progress = _get_rebuild_progress(2, "sataraid1")
    if progress is None:
        progress = _get_rebuild_progress(1, "sataraid1")
    if progress is None:
        progress = "0"

    status_info["status_string"] += "(%s%%)" % (progress,)
    status_info["progress"] = int(progress)


def _get_status_info(category, state):
    """Return status information dict for "category" and "state"."""

    try:
        if category not in STATUS_INFO:
            return None
        if state not in STATUS_INFO[category]:
            return None
        status_info = STATUS_INFO[category][state]

        if category == "raidvol" and state == "rebuilding":
            _set_rebuild_progress(status_info)
        if category == "activerepair" and state == "running":
            _set_active_repair_progress(status_info)
        return status_info
    except:
        return None


def get_status_list():
    """Return list of status information dict."""

    status_list = []
    try:
        statemgr_node = _request(FAL_STATEMGR_PATH,
                                    Entry(ReqTypes.QUERY, STATEMGR_NAME))
        for state_node in statemgr_node.get_node_list(STATE_PATH):
            try:
                category = state_node.get_attr_value(CATEGORY_ATTR)
                state = state_node.get_text()
                status_info = _get_status_info(category, state)
                if status_info:
                    status_list.append(status_info)
            except:
                # Ignore error
                pass
    except:
        # Ignore error
        pass

    return status_list
