# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division

import os
import datetime
import time
from django.utils.translation import ugettext as _
from django import forms
from django.forms.formsets import formset_factory
from django.conf import settings
from project.view import FuncView
from ..fields import (
    RemoteLink3ClientEntrysMultipleCheckboxField,
    RemoteLink3TerminalOwnersEmailTextField,
    RemoteLink3TerminalLabelTextField,
)
from project.fal.remotelink3 import FALRemoteLink3, get_message, logging


def _get_clients():
    clients = []
    try:
        for c in FALRemoteLink3().obtain_clients()["data"]["connections"]:
            clients.append({
                'id': c['id'],
                # 'cid': c['cid'],
                'cid': c['cid'][:5] + '********' + c['cid'][-5:],
                'alloweddenied': c['alloweddenied'],
                'connected_username': c['connected_username'],
                'connected_datetime': c['connected_datetime'],
            })
    except:
        pass
    return clients


# def _update(id, value):
#     try:
#         response = FALRemoteLink3().update_connection_alloweddenied(id, value)
#         success = response['success']
#         message = response['message'] or u""
#         return success, message
#     except Exception as err:
#         return False, u""


def _allow(id):
    # return _update(id, 'allowed')
    try:
        response = FALRemoteLink3().allow_connection(id)
        success = response['success']
        message = response['message'] or u""
        return success, message
    except Exception as err:
        return False, u""


def _deny(id):
    # return _update(id, 'denied')
    try:
        response = FALRemoteLink3().deny_connection(id)
        success = response['success']
        message = response['message'] or u""
        return success, message
    except Exception as err:
        return False, u""


def _remove(id):
    try:
        response = FALRemoteLink3().remove_connection(id)
        success = response['success']
        message = response['message'] or u""
        return success, message
    except Exception as err:
        return False, u""


def _clientlimit_enabled():
    try:
        return FALRemoteLink3().load()["data"]['remoteaccess']['clientlimit']['enabled']
    except:
        return False


class RemoteLink3ClientManagementForm(forms.Form):

    def __init__(self, clients, *args, **kwargs):
        super(RemoteLink3ClientManagementForm, self).__init__(*args, **kwargs)
        choices = [(c['id'], c['id']) for c in clients]
        self.fields['clients'] = RemoteLink3ClientEntrysMultipleCheckboxField(choices=choices)


class RemoteLink3TerminalManagementView(FuncView):

    def __deco_view(view_func):
        def _view_func(self, request):
            template, dictionary = view_func(self, request)
            return {
                self.KEY_TEMPLATE: template,
                self.KEY_TEMPLATE_DIC: dictionary,
            }
        return _view_func

    def __get_initial(self):
        try:
            return [
            ]
        except:
            return []

    views = ['allow', 'deny', 'remove']

    @__deco_view
    def view(self, request):
        clients = _get_clients()
        form = RemoteLink3ClientManagementForm(clients)
        for c, cb in zip(clients, form['clients']):
            c['cb'] = cb

        template = os.path.join(self.TEMPLATES, 'clients.html')
        dictionary = {
            'form': form,
            'clients': clients,
        }
        return template, dictionary

    @__deco_view
    def view_allow(self, request):
        clients = _get_clients()
        form = RemoteLink3ClientManagementForm(clients, data=request.POST)
        for c, cb in zip(clients, form['clients']):
            c['cb'] = cb

        with self.processing(constname='ERR_RL3_STATE_CHANGE') as proc:
            proc.break_if_state_change()
            if form.is_valid():
                ids = form.cleaned_data['clients']
                result = True
                for _id in ids:
                    r, m = _allow(_id)
                    result = result and r
                    self.set_message(m)
                proc.result(result)

        clients = _get_clients()
        form = RemoteLink3ClientManagementForm(clients)
        for e, cb in zip(clients, form['clients']):
            e['cb'] = cb

        template = os.path.join(self.TEMPLATES, 'clients.html')
        dictionary = {
            'form': form,
            'clients': clients,
        }
        return template, dictionary

    @__deco_view
    def view_deny(self, request):
        clients = _get_clients()
        form = RemoteLink3ClientManagementForm(clients, data=request.POST)
        for c, cb in zip(clients, form['clients']):
            c['cb'] = cb

        with self.processing(constname='ERR_RL3_STATE_CHANGE') as proc:
            proc.break_if_state_change()
            if form.is_valid():
                ids = form.cleaned_data['clients']
                result = True
                for _id in ids:
                    r, m = _deny(_id)
                    result = result and r
                    self.set_message(m)
                proc.result(result)

        clients = _get_clients()
        form = RemoteLink3ClientManagementForm(clients)
        for e, cb in zip(clients, form['clients']):
            e['cb'] = cb

        template = os.path.join(self.TEMPLATES, 'clients.html')
        dictionary = {
            'form': form,
            'clients': clients,
        }
        return template, dictionary

    @__deco_view
    def view_remove(self, request):
        clients = _get_clients()
        form = RemoteLink3ClientManagementForm(clients, data=request.POST)
        for c, cb in zip(clients, form['clients']):
            c['cb'] = cb

        with self.processing(constname='ERR_RL3_STATE_CHANGE') as proc:
            proc.break_if_state_change()
            if form.is_valid():
                ids = form.cleaned_data['clients']
                result = True
                for _id in ids:
                    r, m = _remove(_id)
                    result = result and r
                    self.set_message(m)
                proc.result(result)

        clients = _get_clients()
        form = RemoteLink3ClientManagementForm(clients)
        for e, cb in zip(clients, form['clients']):
            e['cb'] = cb

        template = os.path.join(self.TEMPLATES, 'clients.html')
        dictionary = {
            'form': form,
            'clients': clients,
        }
        return template, dictionary

