# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
from django.utils.translation import ugettext as _
from django import forms
from django.conf import settings
from project.view import FuncView
from project.fal.remotelink3 import FALRemoteLink3
from .. fields import RemoteLink3EmailTextField
from fal_info import Results

def _make_qrcode(pincode):
    path = None
    try:
        directory = os.path.join(settings.MEDIA_ROOT, "img", "pkg", "remotelink3")
        filename = FALRemoteLink3().create_qrcode(pincode, directory)["data"]["filename"]
        path = os.path.join("img", "pkg", "remotelink3", filename)
        return path
    except:
        pass
    return path

def _send_mail(address, pincode):
    messages = {
        Results.OK: ("MSG_RL3_SEND_PINCODE", {"mail_address": address or _(u"remotelink3: pincode management: emailaddress is empty")}),
        "default": ("ERR_RL3_SEND_PINCODE", {"mail_address": address or _(u"remotelink3: pincode management: emailaddress is empty")}),
    }
    try:
        res = FALRemoteLink3().send_pincode(address, pincode, messages)
        success = res['success']
        message = res['message']
        success = success if isinstance(success, bool) else False
        message = message if isinstance(message, basestring) else u''
        return success, message
    except:
        return False, u''

def _get_pincode():
    try:
        return FALRemoteLink3().get_pincode()["data"]["pincode"]
    except:
        return None


def _change_pincode():
    try:
        res = FALRemoteLink3().change_pincode()
        success = res["success"]
        message = res["message"]
        return success, message
    except:
        return False


class RL3PINCodeManagementForm(forms.Form):
    email = RemoteLink3EmailTextField(required=False)


class RL3PINCodeManagementView(FuncView):

    def __deco_view(view_func):
        def _view_func(self, request):
            template, dictionary = view_func(self, request)
            return {
                self.KEY_TEMPLATE: template,
                self.KEY_TEMPLATE_DIC: dictionary,
            }
        return _view_func

    def __get_initial(self):
        try:
            return {
                'email': '',
            }
        except:
            return {}

    def __make_qrcode(self, pincode):
        src = None
        try:
            src = os.path.join(settings.MEDIA_URL, _make_qrcode(pincode))
        except:
            pass
        return src

    # QRCODE_URL -> QRCODE_PATH (STATIC_URLはroなので)
    QRCODE_URL = "%s/img/pkg/remotelink3/qrcode.png" % settings.STATIC_URL
    QRCODE_PATH = os.path.join(settings.TMP, 'remotelink3qrcode.png')

    views = ['send', 'update']

    @__deco_view
    def view(self, request):
        form = RL3PINCodeManagementForm(initial=self.__get_initial())

        pincode = _get_pincode()
        src = self.__make_qrcode(pincode)

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'form': form,
            'pincode': pincode,
            'src': src,
        }
        return template, dictionary

    @__deco_view
    def view_send(self, request):
        form = RL3PINCodeManagementForm(data=request.POST)
        pincode = _get_pincode()
        src = self.__make_qrcode(pincode)

        with self.processing(constname='ERR_RL3_STATE_CHANGE') as proc:
            proc.break_if_state_change()
            if form.is_valid():
                f = form.cleaned_data
                result, message = _send_mail(f['email'], pincode)
                proc.result(result)
                self.set_message(message)

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'form': form,
            'pincode': pincode,
            'src': src,
        }
        return template, dictionary

    @__deco_view
    def view_update(self, request):
        form = RL3PINCodeManagementForm(data=request.POST)

        with self.processing(constname='ERR_RL3_STATE_CHANGE') as proc:
            proc.break_if_state_change()
            if form.is_valid():
                result, message = _change_pincode()
                proc.result(result)
                self.set_message(message)

        pincode = _get_pincode()
        src = self.__make_qrcode(pincode)

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'form': form,
            'pincode': pincode,
            'src': src,
        }
        return template, dictionary
